@@ -39,6 +39,7 @@ class Container(AnyEntity):
     # container API
     container_rtype = None
     container_skiprtypes = ()
+    container_skipetypes = ()
     container_computedrtypes = ()
 
 @cached
@@ -74,7 +75,9 @@ class ContainerProtocol(EntityAdapter):
         # container relation is still unset, let's ask the parent
         parent = self.parent
         if parent:
-            return parent.cw_adapt_to('Container').related_container
+            container = parent.cw_adapt_to('Container').related_container
+            if self.entity.e_schema not in container.container_skipetypes:
+                return container
 
     @property
     def parent(self):
@@ -94,6 +97,71 @@ class ContainerClone(EntityAdapter):
     """
     __regid__ = 'Container.clone'
 
+    def clone(self, original=None):
+        """ entry point
+
+        To get the original container:
+        * either we are given the eid of the original container,
+        * or there exists a .clone_rtype_role tuple on the entity
+
+        At the end, self.entity is the fully cloned container.
+        """
+        if original:
+            if not isinstance(original, int):
+                raise TypeError('.clone original should be an eid')
+            self.orig_container_eid = original
+        else: # assumes entity.clone_rtype_role tuple
+            if self.clone_rtype:
+                rtype, role = self.entity.clone_rtype_role
+                self.orig_container_eid = self.entity.related(rtype, role).rows[0][0]
+            else:
+                raise TypeError('.clone wants the original or a relation to the original')
+
+        internal_rtypes, clonable_etypes = self.container_rtypes_etypes()
+
+        orig_to_clone = {self.orig_container_eid: self.entity.eid}
+        relations = defaultdict(list)
+        cloned_etypes = []
+        for etype in self.clonable_etypes():
+            cloned_etypes.append(etype)
+            for rtype, from_to in self._etype_clone(etype, orig_to_clone).iteritems():
+                relations[rtype].extend(from_to)
+
+        # the container itself is walked: its subject relations have not yet
+        # been collected
+        for rtype, from_to in self._container_relink(orig_to_clone).iteritems():
+            relations[rtype].extend(from_to)
+
+        uncloned_etypes = set(cloned_etypes) - clonable_etypes
+        if uncloned_etypes:
+            self.info('etypes %s were not cloned', uncloned_etypes)
+
+        # let's flush all collected relations
+        self.info('linking (%d relations)', len(relations))
+        for rtype, eids in relations.iteritems():
+            self.info('%s linking %s' %
+                      ('internal' if rtype in internal_rtypes else 'external', rtype))
+            subj_obj = []
+            for subj, obj in eids:
+                subj = orig_to_clone[subj]
+                if obj in orig_to_clone:
+                    # internal relinking, else it is a link
+                    # between internal and external nodes
+                    obj = orig_to_clone[obj]
+                subj_obj.append((subj, obj))
+            self._cw.add_relations([(rtype, subj_obj)])
+
+    @cachedproperty
+    def clone_rtype(self):
+        """ returns the <clone> rtype if it exists
+        (it should be defined as a .clone_rtype_role 2-uple
+        defined on the container entity)
+        """
+        try:
+            return self.entity.clone_rtype_role[0]
+        except (AttributeError, IndexError):
+            return None
+
     def _complete_rql(self, etype):
         """ etype -> rql to fetch all instances from the container """
         if hasattr(self.entity, '_complete_rql'):
@@ -257,8 +325,9 @@ class ContainerClone(EntityAdapter):
         etype = ceschema.type
         etype_rql = 'Any X WHERE X is %s, X eid %s' % (
                 etype, self.orig_container_eid)
-        clone_rtype = self.entity.clone_rtype_role[0]
-        skiprtypes = set((clone_rtype,)).union(self.entity.clone_rtypes_to_skip)
+        skiprtypes = set(self.entity.clone_rtypes_to_skip)
+        if self.clone_rtype:
+            skiprtypes.add(self.clone_rtype)
         for rschema in ceschema.subject_relations():
             if rschema.final:
                 continue
@@ -286,47 +355,6 @@ class ContainerClone(EntityAdapter):
                                                  skiprtypes=containerclass.container_skiprtypes)
         return rtypes, etypes
 
-    def _init_clone_map(self):
-        rtype, role = self.entity.clone_rtype_role
-        self.orig_container_eid = self.entity.related(rtype, role).rows[0][0]
-        return {self.orig_container_eid: self.entity.eid}
-
-    def clone(self):
-        """ entry point """
-        internal_rtypes, clonable_etypes = self.container_rtypes_etypes()
-
-        orig_to_clone = self._init_clone_map()
-        relations = defaultdict(list)
-        cloned_etypes = []
-        for etype in self.clonable_etypes():
-            cloned_etypes.append(etype)
-            for rtype, from_to in self._etype_clone(etype, orig_to_clone).iteritems():
-                relations[rtype].extend(from_to)
-
-        # the container itself is walked: its subject relations have not yet
-        # been collected
-        for rtype, from_to in self._container_relink(orig_to_clone).iteritems():
-            relations[rtype].extend(from_to)
-
-        uncloned_etypes = set(cloned_etypes) - clonable_etypes
-        if uncloned_etypes:
-            self.info('etypes %s were not cloned', uncloned_etypes)
-
-        # let's flush all collected relations
-        self.info('linking (%d relations)', len(relations))
-        for rtype, eids in relations.iteritems():
-            self.info('%s linking %s' %
-                      ('internal' if rtype in internal_rtypes else 'external', rtype))
-            subj_obj = []
-            for subj, obj in eids:
-                subj = orig_to_clone[subj]
-                if obj in orig_to_clone:
-                    # internal relinking, else it is a link
-                    # between internal and external nodes
-                    obj = orig_to_clone[obj]
-                subj_obj.append((subj, obj))
-            self._cw.add_relations([(rtype, subj_obj)])
-
     @cachedproperty
     def _specially_handled_rtypes(self):
         """ rtypes in this set will not be handled by the default
